// ------------------------------------------------------------------------------
//  Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
// ------------------------------------------------------------------------------

// **NOTE** This file was generated by a tool and any changes will be overwritten.
// <auto-generated/>

// Template Source: EntityType.cs.tt

namespace Microsoft.Graph
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Text.Json.Serialization;

    /// <summary>
    /// The type Message.
    /// </summary>
    [JsonConverter(typeof(DerivedTypeConverter<Message>))]
    public partial class Message : OutlookItem
    {
    
        ///<summary>
        /// The Message constructor
        ///</summary>
        public Message()
        {
            this.ODataType = "microsoft.graph.message";
        }

        /// <summary>
        /// Gets or sets bcc recipients.
        /// The Bcc: recipients for the message.
        /// </summary>
        [JsonPropertyName("bccRecipients")]
        public IEnumerable<Recipient> BccRecipients { get; set; }
    
        /// <summary>
        /// Gets or sets body.
        /// The body of the message. It can be in HTML or text format. Find out about safe HTML in a message body.
        /// </summary>
        [JsonPropertyName("body")]
        public ItemBody Body { get; set; }
    
        /// <summary>
        /// Gets or sets body preview.
        /// The first 255 characters of the message body. It is in text format.
        /// </summary>
        [JsonPropertyName("bodyPreview")]
        public string BodyPreview { get; set; }
    
        /// <summary>
        /// Gets or sets cc recipients.
        /// The Cc: recipients for the message.
        /// </summary>
        [JsonPropertyName("ccRecipients")]
        public IEnumerable<Recipient> CcRecipients { get; set; }
    
        /// <summary>
        /// Gets or sets conversation id.
        /// The ID of the conversation the email belongs to.
        /// </summary>
        [JsonPropertyName("conversationId")]
        public string ConversationId { get; set; }
    
        /// <summary>
        /// Gets or sets conversation index.
        /// Indicates the position of the message within the conversation.
        /// </summary>
        [JsonPropertyName("conversationIndex")]
        public byte[] ConversationIndex { get; set; }
    
        /// <summary>
        /// Gets or sets flag.
        /// The flag value that indicates the status, start date, due date, or completion date for the message.
        /// </summary>
        [JsonPropertyName("flag")]
        public FollowupFlag Flag { get; set; }
    
        /// <summary>
        /// Gets or sets from.
        /// The owner of the mailbox from which the message is sent. In most cases, this value is the same as the sender property, except for sharing or delegation scenarios. The value must correspond to the actual mailbox used. Find out more about setting the from and sender properties of a message.
        /// </summary>
        [JsonPropertyName("from")]
        public Recipient From { get; set; }
    
        /// <summary>
        /// Gets or sets has attachments.
        /// Indicates whether the message has attachments. This property doesn't include inline attachments, so if a message contains only inline attachments, this property is false. To verify the existence of inline attachments, parse the body property to look for a src attribute, such as &amp;lt;IMG src='cid:image001.jpg@01D26CD8.6C05F070'&amp;gt;.
        /// </summary>
        [JsonPropertyName("hasAttachments")]
        public bool? HasAttachments { get; set; }
    
        /// <summary>
        /// Gets or sets importance.
        /// </summary>
        [JsonPropertyName("importance")]
        public Importance? Importance { get; set; }
    
        /// <summary>
        /// Gets or sets inference classification.
        /// </summary>
        [JsonPropertyName("inferenceClassification")]
        public InferenceClassificationType? InferenceClassification { get; set; }
    
        /// <summary>
        /// Gets or sets internet message headers.
        /// </summary>
        [JsonPropertyName("internetMessageHeaders")]
        public IEnumerable<InternetMessageHeader> InternetMessageHeaders { get; set; }
    
        /// <summary>
        /// Gets or sets internet message id.
        /// </summary>
        [JsonPropertyName("internetMessageId")]
        public string InternetMessageId { get; set; }
    
        /// <summary>
        /// Gets or sets is delivery receipt requested.
        /// </summary>
        [JsonPropertyName("isDeliveryReceiptRequested")]
        public bool? IsDeliveryReceiptRequested { get; set; }
    
        /// <summary>
        /// Gets or sets is draft.
        /// </summary>
        [JsonPropertyName("isDraft")]
        public bool? IsDraft { get; set; }
    
        /// <summary>
        /// Gets or sets is read.
        /// </summary>
        [JsonPropertyName("isRead")]
        public bool? IsRead { get; set; }
    
        /// <summary>
        /// Gets or sets is read receipt requested.
        /// </summary>
        [JsonPropertyName("isReadReceiptRequested")]
        public bool? IsReadReceiptRequested { get; set; }
    
        /// <summary>
        /// Gets or sets parent folder id.
        /// </summary>
        [JsonPropertyName("parentFolderId")]
        public string ParentFolderId { get; set; }
    
        /// <summary>
        /// Gets or sets received date time.
        /// </summary>
        [JsonPropertyName("receivedDateTime")]
        public DateTimeOffset? ReceivedDateTime { get; set; }
    
        /// <summary>
        /// Gets or sets reply to.
        /// </summary>
        [JsonPropertyName("replyTo")]
        public IEnumerable<Recipient> ReplyTo { get; set; }
    
        /// <summary>
        /// Gets or sets sender.
        /// </summary>
        [JsonPropertyName("sender")]
        public Recipient Sender { get; set; }
    
        /// <summary>
        /// Gets or sets sent date time.
        /// </summary>
        [JsonPropertyName("sentDateTime")]
        public DateTimeOffset? SentDateTime { get; set; }
    
        /// <summary>
        /// Gets or sets subject.
        /// </summary>
        [JsonPropertyName("subject")]
        public string Subject { get; set; }
    
        /// <summary>
        /// Gets or sets to recipients.
        /// </summary>
        [JsonPropertyName("toRecipients")]
        public IEnumerable<Recipient> ToRecipients { get; set; }
    
        /// <summary>
        /// Gets or sets unique body.
        /// </summary>
        [JsonPropertyName("uniqueBody")]
        public ItemBody UniqueBody { get; set; }
    
        /// <summary>
        /// Gets or sets web link.
        /// </summary>
        [JsonPropertyName("webLink")]
        public string WebLink { get; set; }
    
        /// <summary>
        /// Gets or sets attachments.
        /// The fileAttachment and itemAttachment attachments for the message.
        /// </summary>
        [JsonPropertyName("attachments")]
        public IMessageAttachmentsCollectionPage Attachments { get; set; }

        /// <summary>
        /// Gets or sets attachmentsNextLink.
        /// </summary>
        [JsonPropertyName("attachments@odata.nextLink")]
        [JsonConverter(typeof(NextLinkConverter))]
        public string AttachmentsNextLink { get; set; }
    
        /// <summary>
        /// Gets or sets extensions.
        /// The collection of open extensions defined for the message. Nullable.
        /// </summary>
        [JsonPropertyName("extensions")]
        public IMessageExtensionsCollectionPage Extensions { get; set; }

        /// <summary>
        /// Gets or sets extensionsNextLink.
        /// </summary>
        [JsonPropertyName("extensions@odata.nextLink")]
        [JsonConverter(typeof(NextLinkConverter))]
        public string ExtensionsNextLink { get; set; }
    
        /// <summary>
        /// Gets or sets multi value extended properties.
        /// The collection of multi-value extended properties defined for the message. Nullable.
        /// </summary>
        [JsonPropertyName("multiValueExtendedProperties")]
        public IMessageMultiValueExtendedPropertiesCollectionPage MultiValueExtendedProperties { get; set; }

        /// <summary>
        /// Gets or sets multiValueExtendedPropertiesNextLink.
        /// </summary>
        [JsonPropertyName("multiValueExtendedProperties@odata.nextLink")]
        [JsonConverter(typeof(NextLinkConverter))]
        public string MultiValueExtendedPropertiesNextLink { get; set; }
    
        /// <summary>
        /// Gets or sets single value extended properties.
        /// The collection of single-value extended properties defined for the message. Nullable.
        /// </summary>
        [JsonPropertyName("singleValueExtendedProperties")]
        public IMessageSingleValueExtendedPropertiesCollectionPage SingleValueExtendedProperties { get; set; }

        /// <summary>
        /// Gets or sets singleValueExtendedPropertiesNextLink.
        /// </summary>
        [JsonPropertyName("singleValueExtendedProperties@odata.nextLink")]
        [JsonConverter(typeof(NextLinkConverter))]
        public string SingleValueExtendedPropertiesNextLink { get; set; }
    
    }
}

